#!/usr/bin/env python3
import argparse
import configparser
from enum import Enum
from cnaas_sync.nav import NAV
from cnaas_sync.ni import NI
from cnaas_sync.nms import NMS


class Target(Enum):
    NI = "ni"
    NAV = "nav"
    ALL = "all"

    @staticmethod
    def show(self):
        print(Target.NI)
        print(Target.NAV)
        print(Target.ALL)


def get_args():
    """
    This function is used  by argpase to parse arguments from stdin and
    convert it to simple manipulable key-value pairs used in the run function
    :return:
    """
    target_options = [x.value for x in list(Target)]
    parser = argparse.ArgumentParser(
        prog="cnaas-sync-cli",
        description="Scripts used for syncing the switches inventory database between NI and NAV with data from NMS",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )

    parser.add_argument(
        "-t",
        "--target",
        choices=target_options,
        metavar=f"{'|'.join(target_options)}",
        required=True,
        help="Target subject of the sync operation. Use all for updating all targets",
    )

    parser.add_argument(
        "-cf",
        "--config-file",
        metavar="file",
        required=True,
        type=argparse.FileType("r"),
        help="Configuration file containing key and values pair for optional parameters",
    )

    parser.add_argument(
        "-nvu",
        "--nav-base-url",
        help="Nav Api base url",
        metavar="str",
        default=None,
        type=str,
    )

    parser.add_argument(
        "-nvk",
        "--nav-api-key",
        help="Nav api key",
        metavar="str",
        default=None,
        type=str,
    )

    parser.add_argument(
        "-niu",
        "--ni-base-url",
        help="Nav Api base url",
        metavar="str",
        default=None,
        type=str,
    )

    parser.add_argument(
        "-niuser",
        "--ni-user-name",
        help="Ni username generated from the django admin or postgres db",
        metavar="str",
        default=None,
        type=str,
    )

    parser.add_argument(
        "-nik",
        "--ni-api-key",
        help="Ni user api-key from django admin",
        metavar="str",
        default=None,
        type=str,
    )

    parser.add_argument(
        "-nmsjwt",
        "--nms-jwt-token",
        help="NMS jwt token",
        metavar="str",
        default=None,
        type=str,
    )

    parser.add_argument(
        "-nmsu",
        "--nms-base-url",
        help="NMS api base url",
        metavar="str",
        default=None,
        type=str,
    )

    parser.add_argument(
        "-nmsid",
        "--nms-client-id",
        help="NMS client id",
        metavar="str",
        default=None,
        type=str,
    )

    parser.add_argument(
        "-nmssecret",
        "--nms-client-secret",
        help="NMS client secret",
        metavar="str",
        default=None,
        type=str,
    )

    parser.add_argument(
        "-nmsendpoint",
        "--nms-client-endpoint",
        help="NMS client endpoint",
        metavar="str",
        default=None,
        type=str,
    )

    parser.add_argument(
        "-v", "--verbose", help="Set verbosity", default=False, action="store_true"
    )

    return parser.parse_args()


def coalesce(value, alternative):
    return alternative if value is None else value


def section_has_key(section, key):
    return key in section


def handle_param_errors(param_dict):
    for key, param in zip(param_dict.keys(), param_dict.values()):
        if len(str(param).strip()) == 0:
            key = str(key).replace("_", "-")
            return (
                True,
                f"Parameter {key} should not be empty. Update the config file or set it with appropriate flag.",
            )
    return False, None


def get_nms(args, config, verbose):
    # Parameters init
    nms_conf = config["NMS"]
    nms_base_url = coalesce(args.nms_base_url, nms_conf["NMS_BASE_URL"])
    # Safeguard all the variables because both args & .ini can be none
    nms_jwt_token = (
        nms_conf["NMS_JWT_TOKEN"]
        if section_has_key(section=nms_conf, key="NMS_JWT_TOKEN")
        else None
    )
    nms_client_id = (
        nms_conf["NMS_OAUTH_CLIENT_ID"]
        if section_has_key(section=nms_conf, key="NMS_OAUTH_CLIENT_ID")
        else None
    )
    nms_client_secret = (
        nms_conf["NMS_OAUTH_CLIENT_SECRET"]
        if section_has_key(section=nms_conf, key="NMS_OAUTH_CLIENT_SECRET")
        else None
    )
    nms_token_endpoint = (
        nms_conf["NMS_OAUTH_TOKEN_ENDPOINT"]
        if section_has_key(section=nms_conf, key="NMS_OAUTH_TOKEN_ENDPOINT")
        else None
    )

    nms_jwt_token = coalesce(args.nms_jwt_token, nms_jwt_token)
    nms_client_id = coalesce(args.nms_client_id, nms_client_id)
    nms_client_secret = coalesce(args.nms_client_secret, nms_client_secret)
    nms_token_endpoint = coalesce(args.nms_client_endpoint, nms_token_endpoint)

    # Check parameters
    param_dict = {
        "nms_base_url": nms_base_url,
        "nms_client_id": nms_client_id,
        "nms_client_secret": nms_client_secret,
        "nms_token_endpoint": nms_token_endpoint,
    }
    if nms_jwt_token:
        param_dict = {
            "nms_base_url": nms_base_url,
            "nms_jwt_token": nms_jwt_token,
        }
    error, msg = handle_param_errors(param_dict)
    if error:
        raise Exception(msg)

    # Init repository
    nms = NMS(
        nms_client_id,
        nms_client_secret,
        nms_token_endpoint,
        nms_base_url,
        nms_jwt_token=nms_jwt_token,
        verbose=verbose,
    )
    return nms


def update_ni(args, config, verbose):
    # Parameters init
    ni_conf = config["NI"]
    ni_base_url = coalesce(args.ni_base_url, ni_conf["NI_BASE_URL"])
    ni_user_name = coalesce(args.ni_user_name, ni_conf["NI_USERNAME"])
    ni_api_key = coalesce(args.ni_api_key, ni_conf["NI_USER_API_KEY"])

    param_dict = {
        "ni_base_url": ni_base_url,
        "ni_user_name": ni_user_name,
        "ni_api_key": ni_api_key,
    }
    error, msg = handle_param_errors(param_dict)
    if error:
        raise Exception(msg)

    # System update
    ni = NI(ni_base_url, ni_user_name, ni_api_key, verbose=verbose)
    nms = get_nms(args, config, verbose)
    nms_devices = nms.get_all_devices()

    # Update NI
    ni.update(nms_devices)


def update_nav(args, config, verbose):
    # Parameters init
    nav_conf = config["NAV"]
    ni_conf = config["NI"]
    nav_base_url = coalesce(args.nav_base_url, nav_conf["NAV_BASE_URL"])
    nav_api_key = coalesce(args.nav_api_key, nav_conf["NAV_API_KEY"])
    ni_base_url = coalesce(args.ni_base_url, ni_conf["NI_BASE_URL"])
    ni_user_name = coalesce(args.ni_user_name, ni_conf["NI_USERNAME"])
    ni_api_key = coalesce(args.ni_api_key, ni_conf["NI_USER_API_KEY"])

    param_dict = {
        "nav_base_url": nav_base_url,
        "nav_api_key": nav_api_key,
        "ni_base_url": ni_base_url,
        "ni_user_name": ni_user_name,
        "ni_api_key": ni_api_key,
    }

    error, msg = handle_param_errors(param_dict)
    if error:
        raise Exception(msg)

    # System update
    ni = NI(ni_base_url, ni_user_name, ni_api_key, verbose=verbose)
    nms = get_nms(args, config, verbose)
    nav = NAV(nav_base_url, nav_api_key, verbose=verbose)
    ni_devices = ni.get_all_devices()

    # Update NAV
    nav.update(ni_devices, nms)


def update_all(args, config, verbose):
    error = False
    try:
        print("Updating NI...")
        update_ni(args, config, verbose)
    except Exception as err:
        error = True
        print("ERROR: Could not update NI")
        print(str(err))

    try:
        print("Updating NAV...")
        update_nav(args, config, verbose)
    except Exception as err:
        error = True
        print("ERROR: Could not update NAV")
        print(str(err))

    if not error:
        print("All updates done!")


def run():
    args = get_args()
    config = configparser.ConfigParser()

    # Required arguments
    target_system = args.target
    config_file = args.config_file
    config.read(config_file.name)

    # Update system
    if target_system == "ni":
        update_ni(args, config, verbose=args.verbose)
    elif target_system == "nav":
        update_nav(args, config, verbose=args.verbose)
    elif target_system == "all":
        update_all(args, config, verbose=args.verbose)


if __name__ == "__main__":
    run()
